
% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Script_Refinery.m
% -------------------------------------------------------------------------
% Description 
%
% This script reproduces the results and graphs produced within the case
% study on the refinery data set.
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2016-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2016 Kris Villez
%
% This file is part of a case study in Matlab/Octave. 
% 
% This program is free software: you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your 
% option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

clc
clear all
close all

currentpath =cd ;
cd ..
cd ..
addpath(genpath([cd '\Adopted']))
addpath(genpath([cd '\SCS_Tools']))
cd(currentpath)

% =========================================================================

A=sprintf([	'Computation of all results is time-consuming. Make sure to \n' ...
            'allow sufficient computing time. Hit any key to continue and \n' ...
            'start computation.']);
disp(A)
pause

fontsize =14 ;

% ============================================================
disp('0. Load and plot data:')

data	=   dlmread('data_refinery.dat') ;
        
t       =	data(:,1)	;
y       =	data(:,3)	;
y0      =	y(1)        ;
y       =	y-y0        ;

t       =	t(:)        ;
y       =	y(:)        ;

figure(1), hold on
    plot(t,y0+y,'k.')
    set(gca,'Xlim',[t(1) t(end)])
    plot(-t-10,y0+y,'ro')
    plot(-t-10,y0+y,'r-')
    plot(-t-10,y0+y,'b--')
    plot(-t-10,y0+y,'k-')
    legend({'Data','\theta Upper bound','f(\beta,x) Upper bound','f(\beta,x) Lower bound','f(\beta,x) Optimal'},'Location','NorthWest')
    text(200,y0+y(end),'D','fontsize', fontsize)

plottime = (1:.01:length(y))-1;
drawnow

% =========================================================================


dt          =   0.05                ;
A=sprintf([ '1. An SCS with a transition located on a grid with steps of \n' ...
            num2str(dt) ' along the function domain will now be fitted. \n' ...
            'This takes some time. I suggest getting a coffee while waiting \n' ...
            'for the results. Please hit any key to continue.']);
disp(A)
pause
 

KnotDist	=	2       ;
Lambda      =	0       ;
Order       =   4       ;
string      =	'FC'	;

% ------ Set 1
Solutions   =   t(1):dt:t(end)      ;
Solutions   =   Solutions(2:end-1)  ;
nSol        =   length(Solutions)   ;
J1          =   nan(nSol,1)         ;

for iSol=1:1:nSol
    if or(mod(iSol-1,50)==0,iSol==nSol)
        disp([' Evaluating grid point ' num2str(iSol) ' of ' num2str(nSol) ' points.'])
    end
    model       =   AutoSCS(y,string,Order,Lambda,KnotDist,Solutions(iSol)+1,[],false) ;
    J1(iSol)    =	model.J                                                     ;
    
end
Solutions1  =   Solutions           ;

% ------ Set 2
Solutions   =   65:dt:73            ;
nSol        =   length(Solutions)	;
J2          =   nan(nSol,1)         ;
disp(['Now a grid of points for a zoon-in plot is evaluated.'])
for iSol=1:1:nSol
    if or(mod(iSol-1,50)==0,iSol==nSol)
        disp([' Evaluating grid point ' num2str(iSol) ' of ' num2str(nSol) ' points.'])
    end
    model       =   AutoSCS(y,string,Order,Lambda,KnotDist,Solutions(iSol)+1,[],false)	;
    J2(iSol)	=   model.J                                                     ;
    
end
Solutions2  =   Solutions           ;

figure(2), 
axes1 = axes('Position',[0.14 0.14 0.85 .825]);
hold on,
    plot(Solutions1,J1,'k.')
    set(gca,'fontsize', fontsize)
    set(gca,'Yscale','log','fontsize', fontsize,'xtick',[0:20:200],'Tickdir','out')
    set(gca,'Xlim',[t(1) t(end)])
% =========================================================================

disp('2. Evaluating bounds to the objective function in continguous intervals') 
disp('Please hit any key to execute.')
pause

tk      =	[	t(1:KnotDist:end-1)	;	t(end)  ]	;

Model	=       CreateSCS(y(:),t,tk,Order,Lambda)	;

String	=       'FC'                                ;
Model	=       ApplyQR(Model,String)               ;
    

Intervals           =   [   0:5	;	1:6	]'*32+t(1)	;
Intervals(end,2)    =       t(end)                  ;
nInt                =       size(Intervals,1)       ;
Bounds              =       nan(nInt,2)             ;
Transitions         =       nan(nInt,1)             ;

for iInt = 1:nInt
    
    [LB,UB,Trans,modelUB,modelLB] = BoundSCS(Model,Intervals(iInt,:)',[],false,false) ;
    
    yLB     =	EvalSCS(modelLB,plottime)           ;
    yUB     =	EvalSCS(modelUB,plottime)           ;
    yTrans  =	EvalSCS(modelUB,Trans)              ;
    
    shift	=	0-1.5*iInt                          ;
    
    Transitions(iInt,:) =       Trans               ;
    Bounds(iInt,:)      =   [	LB	UB	]           ;
    
    figure(1), hold on
    set(gca,'fontsize', fontsize)
        plot(plottime,y0+yLB+shift,'b--')
        plot(plottime,y0+yUB+shift,'r-')
        plot(Trans,y0+yTrans+shift,'ro')
        text(200,y0+yUB(end)+shift,num2str(iInt),'fontsize', fontsize)
        
end

figure(1), hold on
    xlabel('Sample index (x)','fontsize', fontsize)
    ylabel('Tray Level (y)','fontsize', fontsize)
    set(gca,'Xlim',[t(1) t(end)],'Ylim',[206 221],'fontsize', fontsize,'xtick',[0:20:200],'Tickdir','out')
    drawnow()
    pause(.01)
    
% =========================================================================


disp('3. Finding the optimum with the branch-and-bound algorithm.') 
disp('Please hit any key to execute.')
pause

    options         =   OptimsetBB(Model)   ;
    options.node    =   3                   ;
    options.Graph   =   true               ;
    options.verbose =   1                   ;
    options.resolution=1/32;
    
    nk          =   length(Model.tk)                ;
    nt          =   size(Model.QR,1)-1              ;
    IntX        =   Model.tk([1 nk]'*ones(1,nt))    ;
    DistX       =   []                              ;
    
    [oo,Model]  =   BB(Model,options,IntX,DistX)    ;
    Trans = Model.SolX ;

    axes(axes1),	plot(Trans,Model.J,'ks')
    
    set(gca,'fontsize', fontsize,'xtick',[0:20:200],'Tickdir','out')
    
    plot(Transitions,Bounds(:,2),'ro')
    plot(Intervals',[ Bounds(:,1) Bounds(:,1)]','b-')
    
    rectangle('Position',[Solutions2(1),.8,Solutions2(end)-Solutions2(1),.6])
    %PlotVertical([102.1 181.6],'r--')
    %PlotHorizontal([0.5 22.5],'r--')
    plot([  Solutions2(1) 98.5 ; Solutions2(1) 98.5 ; Solutions2(end) 177.7 ; Solutions2(end) 177.7]',...
            [.8 0.43; 1.4 21; .8 0.5; 1.4 21]','k:')
    
    PlotVertical(Intervals(2:4,1),'k--')
    plot(ones(2,1)*Intervals(5:end,1)',[22.5 10^3]'*ones(1,2),'k--')
    plot(ones(2,1)*Intervals(5:end,1)',[ 10^(-1) 0.5]'*ones(1,2),'k--')
    xlabel('Transition (\theta)','fontsize', fontsize)
    ylabel('Sum-of-Squared Residuals (g(\theta))','fontsize', fontsize)
    legend({'Objective function','Global optimum','Upper bound','Lower bound'},'Location','NorthWest','fontsize', fontsize)
    
    
axes2 = axes('Position',[0.57225 0.27 0.35 .35]);
hold on,
    plot(Solutions2,J2,'k.')
    plot(Trans,Model.J,'ks')
    
    set(gca,'fontsize', fontsize,'xtick',[66:2:74])
    
    set(gca,'Xlim',[Solutions2(1) Solutions2(end)],'Ylim',[.8 1.4])
    set(axes2, 'Box', 'on','Yaxislocation','right','Color','none','Tickdir','out')
    
    %xlabel('Transition (\theta)')
    %ylabel('Sum-of-Squared Residuals (g(\theta))')
    drawnow()
    pause(.01)
    
    
    
    for iX=1:length(Model.SolX)
        disp(['     Model.SolX(' num2str(iX) '):  ' sprintf('%10.4f', Model.SolX(iX)) ])    
    end
    disp(['     Model.J:        ' sprintf('%10.4f', Model.J) ])
    disp('	BB optimization terminated')
    
    
    hanax = get(99,'Children');
    axes(hanax(1))
    set(hanax(1),'Yscale','linear','Ylim',[0 7],'Ytick',[0:7],'Tickdir','out','fontsize', fontsize)
    
    %set(gcf,'Position',[2000 -300 560 1080])
    ylabel('$\sqrt{(g(\theta)}$','fontsize', fontsize,'interpreter','latex')
    
    xlabel('Iteration','fontsize', fontsize)
    %
    legend({'Upper bound','Lower bound'},'Location','East','fontsize', fontsize)
    title('(b)','Position',[25 3.5],'HorizontalAlignment','left','VerticalAlignment','middle','fontsize', fontsize)
    set(gca,'Position',[0.14 0.135 0.8 0.37])
    
    axes(hanax(2))
    xlabel('Transition (\theta)','fontsize', fontsize)
    ylabel('Iteration','fontsize', fontsize)
    set(gca,'fontsize', fontsize)
    ticklabels =get(gca,'Yticklabel');
    ticklabels(2:3:end)={''};
    ticklabels(3:3:end)={''};
    set(gca,'Yticklabel',ticklabels,'xtick',[0:20:200],'Tickdir','out','fontsize', fontsize);
    title('(a)','Position',[t(end) 13.5],'HorizontalAlignment','left','fontsize', fontsize)
    set(gca,'Position',[0.14 0.61 0.8 0.37])
    
    drawnow()
    pause(.01)
    
   figure(1), hold on
    yOpt     =	EvalSCS(Model,plottime) ;
    yTrans  =	EvalSCS(Model,Trans) ;
    
    shift = .7 ;
    
        plot(plottime,y0+yOpt+shift,'k-')
        plot(Trans,y0+yTrans+shift,'ks')
        text(200,y0+yOpt(end)+shift,'O','fontsize', fontsize)
    PlotVertical([Intervals(2:end,1)],'k--')
    drawnow()
    pause(.01)
    
    figure(1)
    savefig(1,'REFINERY-FIG1')
    saveas(1,'REFINERY-FIG1.tiff')
    print -depsc -loose REFINERY-FIG1.eps
    pause(.01)
    
    figure(2)
    savefig(2,'REFINERY-FIG2')
    saveas(2,'REFINERY-FIG2.tiff')
    print -depsc -loose REFINERY-FIG2.eps
    pause(.01)
    
    figure(99)
    savefig(99,'REFINERY-FIG3')
    saveas(99,'REFINERY-FIG3.tiff')
    print -depsc -loose REFINERY-FIG3.eps
    
    return
    
