
% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Example_SCSfitting.m
% -------------------------------------------------------------------------
% Description 
%
% This script showcases the use of the SCS toolbox for fitting of a shape
% constrained spline function to a typical ORP data series.
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2015-07-13
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2016 Kris Villez
%
% This file is part of a case study in Matlab/Octave. 
% 
% This program is free software: you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your 
% option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

clc
clear all
close all

currentpath =cd ;
cd ..
cd ..
addpath(genpath([cd '\Adopted']))
addpath(genpath([cd '\SCS_Tools']))
cd(currentpath)

% =========================================================================

data    =   load(['.\Data\DataGroup1'])	;
Y       =   data.Z(:,1:end-1)           ;
Z       =   data.Z(:,end)               ;

% -------------------------------------------------------------------------
% Setup
Disc        =   [3 3]   ;
Graph       =   false   ;
KnotDist    =   1       ;
Lambda      =   0       ;
Order       =   4       ;
String      =   'EAC'	;

Solution    =   [ 42 102]'+1;

Scale       =   std(Y(:))       ;
Y           =   Y./Scale        ;

y           =   Y(37,:) ;
		
figure(101), plot(y*Scale) ;

fontsize =14;

% -------------------------------------------------------------------------
model      =   AutoSCS(y,String,Order,Lambda,KnotDist,[],Disc,Graph)	;

figure(99)
    FigPrep
    AxisPrep
    xlabel('\theta_1 [min.]')
    ylabel('\theta_2 [min.]')
    TickMinutes = [0:5:85] ;
    set(gca,'Xtick',TickMinutes*6+1,'Xticklabel',TickMinutes,...
            'Ytick',TickMinutes*6+1,'Yticklabel',TickMinutes)

shift = 15;
    xx          =	[0:512]/6 ;
    xxsim       =	xx(1):.1:xx(end)    ;
  
figure(1), 
    FigPrep
subplot(2,1,1)
    AxisPrep
    yhat       =   EvalSCS(model) ;
    yysim      =   EvalSCS(model,xxsim*6+1) ;
    
    
    XlimTime    = [-1 85.33+1] ;
    
    plot(xx,y*Scale,'k.')
    plot(xxsim,yysim*Scale-shift,'k-')
    
    
    gcapos = get(gca,'Position') ;
    gcapos(1) = .14 ;
    gcapos(2) = .5 ;
    gcapos(3) = .85 ;
    gcapos(4) = .475 ;
    set(gca,'Position',gcapos) ;
    
    set(gca,'Xlim',XlimTime,'Xtick',TickMinutes,'Xticklabel',[])
    set(gca,'Ylim',[-140 40],'Ytick',[-140:20:40])
    set(gca,'tickdir','out','XminorTick','on')
    
    ylabel({'\makebox[4in][c]{$ORP$}','\makebox[4in][c]{[mV]}'},'Interpreter','latex')
PlotVertical((model.SolX-1)/6,'k--')
legend({'$\mathbf{y}$','$f(\hat{\beta},\hat{\mathbf{\theta}})$','$\hat{\mathbf{\theta}}$'},'Interpreter','Latex','Location','SouthEast','fontsize',fontsize)
    
subplot(2,1,2)
    AxisPrep
    stem(xx,(y(:)-yhat(:))*Scale,'k.')
    gcapos = get(gca,'Position') ;
    gcapos(1) = .14 ;
    gcapos(3) = .85 ;
    set(gca,'Position',gcapos) ;
    
  %  set(gca,'Ylim',[-0.08 0.08],'Ytick',[-0.08:.02:0.08])
    set(gca,'Xlim',XlimTime,'Xtick',TickMinutes)
    xlabel('Time [min.]')
    
    ylabel({'\makebox[4in][c]{Residuals}','\makebox[4in][c]{$y - f(\beta,\theta)$ [mV]}'},'Interpreter','latex')
    PlotVertical((model.SolX-1)/6,'k--')
    
    disp(model.J*Scale.^2)
    
if ~UsingOctave
 figure(99)
    savefig(99,'ORP-DEMO-FIG1')
    print -depsc -loose ORP-DEMO-FIG1.eps
    pause(.01)
        
 figure(1)
    savefig(1,'ORP-DEMO-FIG2')
    print -depsc -loose ORP-DEMO-FIG2.eps
    pause(.01)
end