
% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Script1_PCA.m
% -------------------------------------------------------------------------
% Description 
%
% This script reproduces results and graphs produced within the case study
% on the ORP data set with shape constraines splines (SCS). 
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2016-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2016 Kris Villez
%
% This file is part of a case study in Matlab/Octave. 
% 
% This program is free software: you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your 
% option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

clc
clear all
close all

A=sprintf('This script computes all principal component analysis (PCA) models \n associated with this case study and produces the corresponding figures. \n These computations are very fast. Press any key to continue.');
disp(A)
pause

addpath(genpath('C:\Z\Research\Code\Tools\Intern\Basic\Spike_B_v1.0\Spike_B'))
addpath(genpath('C:\Z\Research\Code\Tools\Intern\Blackbox\DM_1.0_r20150713\DM_1.0\DM'))

dataref =   {'R1'} ;
n       =   513         ;

reactor =	dataref{1}	;

for group	=	0:2
    
    % =========================================================================
    % GET DATA
    
    switch group
        case 0
            data    =   load(['.\Data\DataGroup1'])	;
            Y       =   data.Z(:,1:end-1)       ;
            Z       =   data.Z(:,end)           ;
            data    =   load(['.\Data\DataGroup2'])	;
            Y       =   [Y ; data.Z(:,1:end-1)  ]     ;
            Z       =   [Z ; data.Z(:,end)      ]     ;
            
        case 1
            data    =   load(['.\Data\DataGroup1'])	;
            Y       =   data.Z(:,1:end-1)       ;
            Z       =   data.Z(:,end)           ;
        case 2
            data    =   load(['.\Data\DataGroup2'])	;
            Y       =   data.Z(:,1:end-1)       ;
            Z       =   data.Z(:,end)           ;
    end
    clear data
    
    % =========================================================================
    % ARRANGE/SELECT DATA
    
    normal              =   find(Z==2)      ;
    normal_cal          =   normal(1:100)   ;
    
    % =========================================================================
    % PCA PHASE I
    
    % -------------------------------------------------------------------------
    % Setup
    Model.Centering.Method	= 	'column-mean'       ;
    Model.Scaling.Method 	= 	'none'              ;
    
    Ycal    =	Y(normal_cal,:)         ;
    Yval    =   Y                       ;
    
    % -------------------------------------------------------------------------
    % Computation of PCs/loading vectors with calibration data
    [Tcal,Model]	=	EstimationPCA(Ycal,Model)           ;
    
    % -------------------------------------------------------------------------
    % Vizualization
    XlimPC = [0 11]+.5;
    XtickPC = floor(XlimPC(1)):ceil(XlimPC(end)) ;
    YtickPC = [0:10:100] ;
    
    figure, FigPrep
    subplot(2,1,1), AxisPrep
    han = bar(Model.Summary.Eigenvalues./sum(Model.Summary.Eigenvalues)*100)    ;
    set(han,'EdgeColor','k','FaceColor',[1 1 1]*.73)
    ylabel({' ','Relative Variance','[%]'})
    
    set(gca,'Xlim',XlimPC,'Xtick',XtickPC,'Xticklabel',[],'tickdir','out')
    set(gca,'Ytick',YtickPC)
    grid on
    gcapos = get(gca,'Position') ;
    gcapos(1)   =	.2	;
    gcapos(2)   =	.56 ;
    gcapos(4)   =	.42	;
    set(gca,'Position',gcapos)
    
    subplot(2,1,2)   , AxisPrep
	relcumvar = cumsum(Model.Summary.Eigenvalues)./sum(Model.Summary.Eigenvalues)*100 ;
	han = bar(relcumvar)    ;
    set(han,'EdgeColor','k','FaceColor',[1 1 1]*.73)
    xlabel('#PC')
    ylabel({'Cumulative','Relative Variance','[%]'})
    
    set(gca,'Xlim',XlimPC,'Xtick',XtickPC,'tickdir','out')
    set(gca,'Ytick',YtickPC)
    grid on
    gcapos = get(gca,'Position') ;
    gcapos(1)= .2 ;
    gcapos(4) = .42 ;
    set(gca,'Position',gcapos)
    
    figure, FigPrep
    TickMinutes = [0:5:85] ;
    XlimTime    = [-1 513] ;
    
    for pc=1:2
    subplot(2,1,pc), AxisPrep, hold on,
        PC = Model.Summary.Basis(:,pc) ;
        PC = PC*sign(PC(end)) ;
        stem(PC,'.','Color',[1 1 1]*.73)
        plot(PC,'k.')
        if pc==1
            set(gca,'Xlim',XlimTime,'Xtick',TickMinutes*6,'Xticklabel',TickMinutes)
            ylabel('PC1 [mV]','Position',[ -73 mean(get(gca,'Ylim'))])
        else
            set(gca,'Xlim',XlimTime,'Xtick',TickMinutes*6,'Xticklabel',TickMinutes)
            ylabel('PC2 [mV]','Position',[ -73 mean(get(gca,'Ylim'))])
        end
    gcapos = get(gca,'Position') ;
    gcapos(1) = .17 ;
    set(gca,'Position',gcapos) ;
    end
    xlabel('Time [min.]')
    
    % =========================================================================
    % PCA PHASE II
    
    % Computation of scores for all data
    [Tval]          =	EstimationPCA(Yval,Model,+1) 		;
    
    % -------------------------------------------------------------------------
    % Dimension reduction (2 PCs)
    Tcal(:,3:end)   =	0                                   ;
    Tval(:,3:end)   =	0                                   ;
    
    % -------------------------------------------------------------------------
    % Reconstruction and statistic computation
    [Yval_rec]      =	EstimationPCA(Tval,Model,-1) 		;
    SSR             =	sum( (Yval_rec-Yval).^2 ,2)         ;
    
    save(['.\Results\SummaryPCA_Group' num2str(group)],'SSR')
    
end

 figure(3)
    savefig(3,'.\Figures\ORP-SUPP-PCA-FIG1')
    print -depsc -loose .\Figures\ORP-SUPP-PCA-FIG1.eps
    pause(.01)
    
 figure(4)
    savefig(4,'.\Figures\ORP-SUPP-PCA-FIG2')
    print -depsc -loose .\Figures\ORP-SUPP-PCA-FIG2.eps
    pause(.01)    
    
 figure(5)
    savefig(5,'.\Figures\ORP-SUPP-PCA-FIG3')
    print -depsc -loose .\Figures\ORP-SUPP-PCA-FIG3.eps
    pause(.01)
    
 figure(6)
    savefig(6,'.\Figures\ORP-SUPP-PCA-FIG4')
    print -depsc -loose .\Figures\ORP-SUPP-PCA-FIG4.eps
    pause(.01)    
return
    