
% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Script2b_SCS_group2.m
% -------------------------------------------------------------------------
% Description 
%
% This script reproduces results and graphs produced within the case study
% on the ORP data set with shape constraines splines (SCS). 
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2016-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2016 Kris Villez
%
% This file is part of a case study in Matlab/Octave. 
% 
% This program is free software: you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your 
% option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

clc
clear all
close all

currentpath =cd ;
cd ..
cd ..
addpath(genpath([cd '\Adopted']))
addpath(genpath([cd '\SCS_Tools']))
cd(currentpath)

% =========================================================================

disp(sprintf('This script identifies all shape constrained spline (SCS) functions for \n the second group of batches included in the case study. These computations \n are very time-consuming so make sure you are running this on a dedicated \n machine or server. For a demo, use ''Example_SCSfitting.m'' instead. Press any \n key to continue.'))
pause

addpath('C:\Z\Research\Code\Tools\Intern\Basic\Spike_B_v1.0\Spike_B')
%addpath('C:\Z\Research\Code\Tools\Intern\Blackbox\Unsupervised\Continuous\LatentVariableModelling')

    %addpath(        '..\SPIKE_B_v1.0\SPIKE_B\'	)
	addpath(genpath('C:\Z\Research\Code\Tools\Intern\Qualitative\SCS_v2.0\SCS\Spike_F_v1.0')	)
	%rmpath(        '..\Spike_O_v1.1\Spike_O\'	)
    addpath('C:\Z\Research\Code\Tools\Intern\Optimization\Spike_O_v1.1\Spike_O')
    addpath('C:\Z\Research\Code\Tools\Intern\Qualitative\SCS_v2.0\SCS')
    
dataref =   {'R1','R5'} ;
n       =   513         ;

reactor =	dataref{1}	;
group	=	2           ;

% =========================================================================
% GET DATA

data    =   load(['.\Data\DataGroup2'])	;
Y       =   data.Z(:,1:end-1)       ;
Z       =   data.Z(:,end)           ;
clear data

% =========================================================================
% ARRANGE/SELECT DATA

batches            =	1:length(Z)            ;
Z                   =   Z(batches,:)   ;


% =========================================================================
% SCS

% -------------------------------------------------------------------------
% Setup
Disc        =   [3 3]   ;
Graph       =   false   ;
KnotDist    =   1       ;
Lambda      =   0       ;
Order       =   4       ;
String      =   'EAC'	;

Solution    =   [ 42 103]'+1;

Scale       =   std(Y(:))       ;
Y           =   Y./Scale        ;

SSR         =   nan(length(batches),2) 	;
M 			=	cell(length(batches),2)	;

SummaryFile 	=	['.\Results\SummarySCS_Group' num2str(group)] ;
try
	load(SummaryFile)
end

% -------------------------------------------------------------------------
% SCS fitting
for row = batches(:)'
    clc
    
    index = find(batches==row);
    
    disp(['Batch index: ' num2str(index)  ] )
    disp(['Batch label: ' num2str(row) ' of ' num2str(length(batches)) ] )
    
	if or(isnan(SSR(row,1)),isinf(SSR(row,1)))
		y           =   Y(index,:)	;
		y           =   y(:)        ;
        
        figure, plot(y)
    
		% -------------------------------------------------------------------------
		model1      =   AutoSCS(y,String,Order,Lambda,KnotDist,Solution,Disc,Graph)	;
		SSR(row,1)	=   model1.J*Scale.^2    ;
		
		% -------------------------------------------------------------------------
		model2      =   AutoSCS(y,String,Order,Lambda,KnotDist,[],Disc,Graph)	;
		SSR(row,2)	=   model2.J*Scale.^2    ;
		
		save(['.\Results\Group' num2str(group) '_Batch' num2str(row) '_SCSmodels'] ,'model1','model2')
	else
		disp('... Already solved - Skipping this batch')
	end
    
	save(SummaryFile,'SSR')
end


return
    