
% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Script3_AnomalyDetection.m
% -------------------------------------------------------------------------
% Description 
%
% This script reproduces results and graphs produced within the case study
% on the ORP data set with shape constraines splines (SCS). 
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2016-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2016 Kris Villez
%
% This file is part of a case study in Matlab/Octave. 
% 
% This program is free software: you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your 
% option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

clc
clear all
close all

currentpath =cd ;
cd ..
cd ..
addpath(genpath([cd '\Adopted']))
cd(currentpath)

% =========================================================================

PathResults     =   '.\Results_Archive\'    ;

disp(sprintf('This script applies evaluates the shape constrained splines (SCS) and \n principal component analysis (PCA) models for anomaly detection and \n produces all corresponding graphs. The results are based on pre-computed \n results available in the folder ''./Results_Archive/''. To use newly \n obtained results, change the folder name at the top of this script to \n ''./Results/''.'))
pause

if ~(exist('legendflex')==2)
    disp(sprintf('\nFor best results, install the function ''legendflex.m''. Obtain it here \n if desired: \n http://www.mathworks.com/matlabcentral/fileexchange/31092-legendflex-m--a-more-flexible--customizable-legend '))
end
if ~(exist('columnlegend')==2)
    disp(sprintf('\nFor best results, install the function ''columnlegend.m''. Obtain it here \n if desired: \n http://www.mathworks.com/matlabcentral/fileexchange/27389-columnlegend '))
end
disp(' ')

FontSize 		= 	14									;
PlotString      =   {'o','o'}                           ;
PlotColor       =   {.37*ones(1,3),'k'}                 ;
GroupColor      =	{'r','b','k'}                       ;
GroupText       =	{'Global','Subset 1','Subset 2'}    ;
MethodText      =	{'SCS','PCA'}                       ;
XlimTime        =	[-1 85.33+1]                        ;

for fignum=100:101
figure(fignum), FigPrep
subplot(2,1,1), hold on,
markerhan3(1) = stem(-10,1,'.','Color',.5*ones(1,3)    )    ;
markerhan3(2) = stem(-10,1,'*','Color',.5*ones(1,3)    )    ;
markerhan3(3) = stem(-10,1,'k.'                        )    ;
markerhan3(4) =   plot(-10,1,'--','Color','k') ;
markerhan3(5) =    plot(-10,1,'o','Color','k')	;
markerhan3(6) =    plot(-10,1,'x','Color','k')	;
subplot(2,1,2), hold on,
end

for Group=(0:2)
    clear SSRS
    clear Alarms
    
    disp(['Group: ' num2str(Group)])
    
    % =========================================================================
    % GET DATA
    switch Group
        case 0 % Group 1 & 2 - same model
            
            data    =   load(['.\Data\DataGroup1'])	;
            Y1      =   data.Z(:,1:end-1)       ;
            Z1      =   data.Z(:,end)           ;
            
            data    =   load(['.\Data\DataGroup2'])	;
            Y2      =   data.Z(:,1:end-1)       ;
            Z2      =   data.Z(:,end)           ;
            
            Y       =	[   Y1  ;   Y2  ]       ;
            Z       =	[   Z1  ;   Z2  ]       ;
            
            clear data Y1 Y2 Z1 Z2
            
            load([PathResults 'SummarySCS_Group1'])
            SSR         =   SSR(:,2)    ;
            SSRS(:,1)	=   SSR         ;
            
            load([PathResults 'SummarySCS_Group2'])
            SSR         =   SSR(:,2)    ;
            SSRS        =   [   SSRS(:,1)	;   SSR	]	;
            
            load([PathResults 'SummaryPCA_Group0'])
            SSRS(:,2)	=   SSR         ;
            
            CycleIndex 	=	(1:size(SSRS,1))'	;
            
        case 1 % Group 1 alone
            data    =   load(['.\Data\DataGroup1'])	;
            Y       =   data.Z(:,1:end-1)       ;
            Z       =   data.Z(:,end)           ;
            clear data
            
            load([PathResults 'SummarySCS_Group1'])
            SSR         =   SSR(:,2)    ;
            SSRS(:,1)	=   SSR         ;
            
            load([PathResults 'SummaryPCA_Group1'])
            SSRS(:,2)	=   SSR         ;
            
            CycleIndex 	=	(1:size(SSRS,1))'	;
            
        case 2 % Group 2 alone
            data    =   load(['.\Data\DataGroup2'])	;
            Y       =   data.Z(:,1:end-1)       ;
            Z       =   data.Z(:,end)           ;
            clear data
            
            load([PathResults 'SummarySCS_Group2'])
            SSR         =   SSR(:,2)    ;
            SSRS(:,1)	=   SSR         ;
            
            load([PathResults 'SummaryPCA_Group2'])
            SSRS(:,2)	=   SSR         ;
            
            CycleIndex 	=	410+(1:size(SSRS,1))'	;
            
    end
    
    %     disp('limits')
    %     min(SSRS(:,1))
    %     min(SSRS(:,2))
    %     max(SSRS(:,1))
    %     max(SSRS(:,2))
    %     disp('limits end')
    
    % =========================================================================
    % ARRANGE/SELECT DATA
    
    normal              =   find(Z==2)      ;
    normal_cal          =   normal(1:100)   ;
    normal_val          =   setdiff(normal,normal_cal)      ;
    abnormal            =   find(Z==1)      ;
    
    % =====================================================================
    if Group==0
        
        for graph=0:1
            fignum	=   20+graph	;
            figure(fignum),	FigPrep, AxisPrep
            
            select  =   80:80:1684      ;
            nselect =	length(select)  ;
            set(gca, 'ColorOrder', 0.89*jet(max(3,nselect)))
            
            Yselect	=   Y(select,:) ;
            
            if graph
                shift = 0 ;
                for i=2:nselect
                    D = Yselect(i-1,:)-Yselect(i,:) ;
                    shift = -min(D) + 10 ;
                    Yselect(i,:) = Yselect(i,:) - shift ;
                end
            end
            hdlY = plot((1:size(Yselect,2))/6,Yselect','.-') ;
            
            legHdl = legend(num2str(CycleIndex(select)),'Location','northeastoutside') ;
            
            set(gca,'Xlim',XlimTime,'Xtick',0:5:XlimTime(end))
            Ylim = [ min(Yselect(:))-42 max(Yselect(:))+42 ];
            set(gca,'Ylim',Ylim)
            ylabel('ORP [mV]','Interpreter','latex')
            xlabel('Time [min.]')
            set(gca,'tickdir','out','XminorTick','on')
            
			if ~UsingOctave
				if graph
					print -depsc -loose .\Figures\ORP-FIG20.eps
				else
					print -depsc -loose .\Figures\ORP-FIG21.eps
				end
			end
        end
    end
    
    
    % =====================================================================
    
    for method = 1:2
        
        
        if method==1
            sub = 1;
        else
            sub = 2;
        end
        
        % ====================================
        % Plot statistic
        if Group==0
            figure(100),
        else
            figure(101),
        end
        
        FigPrep
        subplot(2,1,sub), AxisPrep, hold on
        if UsingOctave
            plot(CycleIndex(normal),	SSRS(normal,method),	'.','Color',.5*ones(1,3)    ),
            if method==2
                plot(CycleIndex(normal_cal), SSRS(normal_cal,method),  '*','Color',.5*ones(1,3)   ),
            end
            plot(CycleIndex(abnormal),	SSRS(abnormal,method),	'k.'                        ),
        else
            stem(CycleIndex(normal),	SSRS(normal,method),	'.','Color',.5*ones(1,3)    ),
            if method==2
                stem(CycleIndex(normal_cal), SSRS(normal_cal,method),  '*','Color',.5*ones(1,3)   ),
            end
            stem(CycleIndex(abnormal),	SSRS(abnormal,method),	'k.'                        ),
        end
        set(gca,'Yscale','log')
            
        % =========================================================================
        % ANOMALY DETECTION
        SSRlimit(method)    =	max(SSRS(normal,method))        ;
        Alarms(:,method)    =   SSRS(:,method)>SSRlimit(method)	;
        
        % ====================================
        % Plot detection limit
            
        LimitStr = '--';
        plot([CycleIndex(1)-.5 CycleIndex(end)+.5],SSRlimit(method)*ones(1,2),LimitStr,'Color','k')
        
        
    end
    
    AlarmsUnique        =	Alarms                          ;
    AlarmsUnique(:,1)	=	and(Alarms(:,1),~Alarms(:,2))	;
    AlarmsUnique(:,2)	=	and(Alarms(:,2),~Alarms(:,1))	;
    
    for method=1:2
        
        
        detection = abnormal(Alarms(abnormal,method))	;
        exclusive = intersect(abnormal(AlarmsUnique(abnormal,method)),abnormal) ;
        
        disp(['     Detection method: ' num2str(method)])
        
        % -------------------------------------------------------------------------
        % Vizualization
        if method==1
            sub = 1;
        else
            sub = 2;
        end
        
        if Group==0
            figure(100), 
        else
            figure(101),
        end
        
        subplot(2,1,sub),
        Level = 10^(7.8);
        plot(CycleIndex(detection),Level*ones(1,length(detection)),'o','Color','k')
        Level = 10^(7.6);
        plot(CycleIndex(exclusive),Level*ones(1,length(exclusive)),'x','Color','k')
        
        for series=1:2
            switch series
                case 1
                    select  =   exclusive	;
                case 2
                    select  =   intersect(abnormal,setdiff(detection,exclusive))	;
            end
            if ~and(series==2,method==2)
                
                fignum	=   Group*4+method+(series-1)*2 ;
                figure(fignum),	FigPrep, AxisPrep
                nselect = length(select) ;
                set(gca, 'ColorOrder', 0.89*jet(max(3,nselect)))
                
                Yselect	=   Y(select,:) ;
                
                shift = 0 ;
                for i=2:nselect
                    D = Yselect(i-1,:)-Yselect(i,:) ;
                    shift = -min(D) + 10 ;
                    Yselect(i,:) = Yselect(i,:) - shift ;
                end
                hdlY = plot((1:size(Yselect,2))/6,Yselect','.-') ;
                
                if length(select)>=23
                    
                    gcapos = get(gca,'Position');
                    gcapos(3) = .6 ;
                    set(gca,'Position',gcapos)
                    if exist('columnlegend','file')==2
                        columnlegend(2, num2str(CycleIndex(select)),'Location','northeastoutside','boxon') ;
                    else
                        legend(num2str(CycleIndex(select)))
                    end
                    
                else
                    legHdl = legend(num2str(CycleIndex(select)),'Location','northeastoutside') ;
                end
                
                set(gca,'Xlim',XlimTime,'Xtick',0:5:XlimTime(end))
                Ylim = [ min(Yselect(:))-42 max(Yselect(:))+42 ];
                set(gca,'Ylim',Ylim)
                ylabel('ORP [mV]','Interpreter','latex')
                xlabel('Time [min.]')
                set(gca,'tickdir','out','XminorTick','on')
                switch series
                    case 1
                        title([GroupText{Group+1} ' - Exclusive detection by ' MethodText{method} ])
                    case 2
                        title([GroupText{Group+1} ' - Detection by both ' MethodText{1} ' and ' MethodText{2}  ])
                end
            end
        end
        
    end
    
    Alarms  = nan(length(Z),2) ;
    SSRlimit = nan(1,2);
    
    for method=1:2
        
        
        SSR = SSRS(:,method) ;
        % =========================================================================
        % ANOMALY DETECTION
        SSRlimit(method)	=	max(SSR(normal))        ;
        Alarms(:,method)    =   SSR>SSRlimit(method)	;
        
        % =========================================================================
        % RECEIVER OPERATING CHARACTERISTIC
        
        % Fawcett, Tom (2006); An introduction to ROC analysis, Pattern Recognition
        % Letters, 27, 861874.
        
        SSRlimits   =   sort(SSR)           ;
        n_limits    =   length(SSRlimits)   ;
        Rates       =   nan(n_limits,2)     ;
        
        for i_limits=1:n_limits
            Rates(i_limits,:)	=	[   sum(SSR(normal_val)>SSRlimits(i_limits))./length(normal_val)     ...
                sum(SSR(abnormal)>SSRlimits(i_limits))./length(abnormal)	];
        end
        
        MS  =   5-method*2	;
        IsRate0         =   (Rates(:,1)==0)             ;
        IndexRate0      =	find(IsRate0)               ;
        [oo,minRate2]	=   max(Rates(IndexRate0,2))	;
        Rates           =	[	Rates(~IsRate0,:) ; Rates(IndexRate0(minRate2),:) ];
        
        IsRate1         =   (Rates(:,2)==1)             ;
        IndexRate1      =	find(IsRate1)               ;
        [oo,maxRate1]	=   min(Rates(IndexRate1,1))	;
        Rates           =   [	Rates(~IsRate1,:) ; Rates(IndexRate1(maxRate1),:) ];
        
        Rates = 100*Rates	;
        
        % =====================================================================
        figure(102), hold on
        stairs(Rates(:,1),Rates(:,2),[GroupColor{Group+1} PlotString{method}],'LineWidth',1.2,'MarkerSize',MS,'MarkerFaceColor',GroupColor{Group+1})
        % =====================================================================
    end
    
    FigPrep
    if Group==0
        han         =   legend({'Normal','Calibration set','Anomalous','Detection limit','SCS detection only','PCA detection only'},'fontsize',FontSize);
        hanpos      =   get(han,'Position')     ;
        hanpos(1:2) =   [   0.60     0.30     ]   ;
        set(han,'Position',hanpos) ;
    end
    
    
    drawnow
end


for fignum=100:101
    figure(fignum)
    for sub = 1:2 %fliplr(1:3)
        subplot(2,1,sub),	AxisPrep
        set(gca,'Xlim',[-1 CycleIndex(end)+2],'Ylim',10.^[2.4 7.8],'Ytick',10.^[2:7.5],'tickdir','out','XminorTick','on')
        
        if fignum==101
            PlotVertical(410.5,'k--')
        end
        
        gcapos      =   get(gca,'Position') ;
        gcapos(3)   =	0.81                ;
        gcapos(4)   =	0.42                ;
        set(gca,'Position',gcapos)          ;
        
        switch sub
            case 1
                set(gca,'Xticklabel',[])
                ylabel({'SSR [mV^2]','SCS'})
                text(CycleIndex(end)+37,10^5,'(a)','FontSize',14)
                
                gcapos      =   get(gca,'Position') ;
                gcapos(2)   =	0.55                ;
                set(gca,'Position',gcapos)          ;
                
                legend({'Normal','Calibration set','Anomalous','Detection limit','Detection','Exclusive detection'},'Location','East')
            case 2
                xlabel('Batch cycle index [-]')
                ylabel({'SSR [mV^2]','PCA'})
                text(CycleIndex(end)+37,10^5,'(b)','FontSize',14)
                
                
                %         case 3
                %             text(CycleIndex(end)+37,10^5,'(c)','FontSize',12)
                
        end
    end
end

figure(102)
FigPrep
plot([0 100],[0 100],'k-')
grid on
axis equal
axis square
set(gca,'Xlim',[-1 101],'Ylim',[-1 101],'Xtick',[0:10:100],'Ytick',[0:10:100],'tickdir','out')
legend({'SCS_{global}','PCA_{global}','SCS_{subset 1}','PCA_{subset 1}','SCS_{subset 2}','PCA_{subset 2}'},'Location','SouthEast')
xlabel('False Positive Rate [%]')
ylabel('True Positive Rate [%]')

drawnow

if ~UsingOctave
	figure(4)

savefig(4,'.\Figures\ORP-FIG1')
print -depsc -loose .\Figures\ORP-FIG1.eps
pause(.01)

figure(100)
savefig(100,'.\Figures\ORP-FIG3a')
print -depsc -loose .\Figures\ORP-FIG3a.eps
pause(.01)

figure(101)
savefig(101,'.\Figures\ORP-FIG3b')
print -depsc -loose .\Figures\ORP-FIG3b.eps
pause(.01)

figure(102)
savefig(102,'.\Figures\ORP-FIG4')
print -depsc -loose .\Figures\ORP-FIG4.eps
pause(.01)

figure(1)
savefig(1,'.\Figures\ORP-SUPP-FIG1')
print -depsc -loose .\Figures\ORP-SUPP-FIG1.eps
pause(.01)

figure(2)
savefig(1,'.\Figures\ORP-SUPP-FIG2')
print -depsc -loose .\Figures\ORP-SUPP-FIG2.eps
pause(.01)

figure(3)
savefig(1,'.\Figures\ORP-SUPP-FIG3')
print -depsc -loose .\Figures\ORP-SUPP-FIG3.eps
pause(.01)

figure(5)
savefig(5,'.\Figures\ORP-SUPP-FIG4')
print -depsc -loose .\Figures\ORP-SUPP-FIG4.eps
pause(.01)

figure(6)
savefig(6,'.\Figures\ORP-SUPP-FIG5')
print -depsc -loose .\Figures\ORP-SUPP-FIG5.eps
pause(.01)

figure(7)
savefig(7,'.\Figures\ORP-SUPP-FIG6')
print -depsc -loose .\Figures\ORP-SUPP-FIG6.eps
pause(.01)

figure(9)
savefig(9,'.\Figures\ORP-SUPP-FIG7')
print -depsc -loose .\Figures\ORP-SUPP-FIG7.eps
pause(.01)

figure(10)
savefig(10,'.\Figures\ORP-SUPP-FIG8')
print -depsc -loose .\Figures\ORP-SUPP-FIG8.eps
pause(.01)

figure(11)
savefig(11,'.\Figures\ORP-SUPP-FIG9')
print -depsc -loose .\Figures\ORP-SUPP-FIG9.eps
pause(.01)
end

return
